/*
 * Common display-related functions.
 *
 * Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "display.h"

/* Provided by the application. */

#include "vga.h"



/* Return the line data position for the given pixel. */

int get_position(int x)
{
    int cell = x / CELL_SIZE, offset = x % CELL_SIZE;
    int pos = (cell / 2) * CELL_SIZE + offset;

    return cell % 2 ? pos + LINE_LENGTH / 2 : pos;
}

/* Provide a pattern to test the line data. */

void test_linedata(uint8_t *framebuffer)
{
    int x, y;
    uint8_t *linedata = framebuffer;

    for (y = 0; y < LINE_COUNT; y++)
    {
        for (x = 0; x < LINE_LENGTH; x++)
        {
            /* Pixel: I0RRGGBB = Y0YYYYXX */

            linedata[get_position(x)] = (x % 2) ?
                    (((y / (LINE_COUNT / 32)) & 0b1) << 7) |
                    (((y / (LINE_COUNT / 16)) & 0b1111) << 2) |
                    ((x / (LINE_LENGTH / 4)) & 0b11) :
                0x00;
        }

        linedata += LINE_LENGTH;
    }
}
